
/******************************************************************************
* MODULE     : int_mod.gen.h
* DESCRIPTION: modular integers
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef INT_MOD_H
#define INT_MOD_H
#include "tree.hh"
#include "category.hh"

template<int p> struct int_mod {
  int i;
  static const int FFT_threshold = 2400;
  inline int_mod  () { i=0; }
  inline int_mod  (int i2) { i= i2%p; }
  inline int_mod  (long long int i2) { i= i2%p; }
  inline int_mod  (const int_mod<p>& i2) { i= i2.i; }
  inline int_mod<p>& operator = (int_mod<p> i2) { i= i2.i; return *this; }
  inline operator int () { return i; }
  operator string ();
  operator tree ();
};

template<int p> struct Karatsuba_threshold<int_mod<p> > : define_int<16> {};
template<int p> struct characteristic<int_mod<p> >      : define_int<p>  {};
template<int p> struct has_divisible<int_mod<p> > : define_bool< (p>1000) > {};

#define TMPL template<int p>
TMPL ostream& operator << (ostream& out, int_mod<p> i);
TMPL int_mod<p> operator + (int_mod<p> i, int_mod<p> j);
TMPL int_mod<p> operator - (int_mod<p> i);
TMPL int_mod<p> operator - (int_mod<p> i, int_mod<p> j);
TMPL int_mod<p> operator * (int_mod<p> i, int_mod<p> j);
TMPL int_mod<p> operator / (int_mod<p> i, int_mod<p> j);

TMPL inline bool operator == (int_mod<p> i, int_mod<p> j) {
  return i.i == j.i; }
TMPL inline bool operator != (int_mod<p> i, int_mod<p> j) {
  return i.i != j.i; }

TMPL inline void operator += (int_mod<p>& i, int_mod<p> j) { i=i+j; }
TMPL inline void operator -= (int_mod<p>& i, int_mod<p> j) { i=i-j; }
TMPL inline void operator *= (int_mod<p>& i, int_mod<p> j) { i=i*j; }
TMPL inline void operator /= (int_mod<p>& i, int_mod<p> j) { i=i/j; }
TMPL inline void operator %= (int_mod<p>& i, int_mod<p> j) { i=i%j; }
#undef TMPL

#include "Arithmetic/int_mod.cc"
#endif // defined INT_MOD_H
