
/******************************************************************************
* MODULE     : rational.gen.h
* DESCRIPTION: rationals based on gmp
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef RATIONAL_H
#define RATIONAL_H
#include "Arithmetic/integer.hh"

struct rational_rep: public concrete_struct {
  integer p, q;
  inline rational_rep (integer p2, integer q2): p(p2), q(q2) {}
  inline ~rational_rep () {}
};

class rational {
  CONCRETE(rational);
public:
  rational ();
  rational (int i);
  rational (integer p, integer q=1);
  operator string ();
  operator tree ();
};
CONCRETE_CODE(rational);

ostream& operator << (ostream& out, rational i);
rational operator + (rational i, rational j);
rational operator - (rational i);
rational operator - (rational i, rational j);
rational operator * (rational i, rational j);
rational operator / (rational i, rational j);

inline bool operator == (rational i, rational j) {
  return i->p*j->q == i->q*j->p; }
inline bool operator != (rational i, rational j) {
  return i->p*j->q != i->q*j->p; }

inline void operator += (rational& i, rational j) { i=i+j; }
inline void operator -= (rational& i, rational j) { i=i-j; }
inline void operator *= (rational& i, rational j) { i=i*j; }
inline void operator /= (rational& i, rational j) { i=i/j; }


#endif // defined RATIONAL_H
