
/******************************************************************************
* MODULE     : complex.gen.h
* DESCRIPTION: complex numbers
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef COMPLEX_H
#define COMPLEX_H
#include <math.h>

template<class R> struct complex {
  R re, im;

  inline complex (): re(0), im(0) {}
  inline complex (int i): re (i), im (0) {}
  inline complex (R x): re (x), im (0) {}
  inline complex (R x, R y): re (x), im (y) {}
  inline complex (const complex<R>& z): re (z.re), im (z.im) {}

  inline complex<R> operator = (complex<R> z) {
    re= z.re; im=z.im; return *this; }
  inline complex<R> operator += (const complex<R>& z) {
    re += z.re; im += z.im; return *this; }
  inline complex<R> operator -= (const complex<R>& z) {
    re -= z.re; im -= z.im; return *this; }

  inline R real () { return re; }
  inline R imag () { return im; }
};

#define TMPL template<class R>
TMPL inline bool operator == (const complex<R>& z1, const complex<R>& z2) {
  return (z1.re==z2.re) && (z1.im==z2.im); }
TMPL inline bool operator != (const complex<R>& z1, const complex<R>& z2) {
  return (z1.re!=z2.re) || (z1.im!=z2.im); }
TMPL inline complex<R> operator + (const complex<R>& z1, const complex<R>& z2) {
  return complex<R> (z1.re+z2.re, z1.im+z2.im); }
TMPL inline complex<R> operator - (const complex<R>& z1) {
  return complex<R> (-z1.re, -z1.im); }
TMPL inline complex<R> operator - (const complex<R>& z1, const complex<R>& z2) {
  return complex<R> (z1.re-z2.re, z1.im-z2.im); }
TMPL inline complex<R> operator * (const complex<R>& z1, R r) {
  return complex<R> (r*z1.re, r*z1.im); }
TMPL inline complex<R> operator * (R r, const complex<R>& z2) {
  return complex<R> (r*z2.re, r*z2.im); }
TMPL inline complex<R> operator * (const complex<R>& z1, const complex<R>& z2) {
  return complex<R> (z1.re*z2.re- z1.im*z2.im, z1.re*z2.im+ z1.im*z2.re); }
TMPL inline complex<R> operator / (const complex<R>& z1, R r) {
  return complex<R> (z1.re/r, z1.im/r); }
TMPL inline complex<R> operator / (R r, const complex<R>& z2) {
  R sq= z2.re*z2.re+ z2.im*z2.im;
  return complex<R> ((r*z2.re)/sq, (-r*z2.im)/sq); }
TMPL inline complex<R> operator / (const complex<R>& z1, const complex<R>& z2) {
  R sq= z2.re*z2.re+ z2.im*z2.im;
  R x = z1.re*z2.re+ z1.im*z2.im;
  R y = z1.im*z2.re- z1.re*z2.im;
  return complex<R> (x/sq, y/sq); }
TMPL inline R abs (const complex<R>& z) {
  return hypot (z.re, z.im); }
TMPL inline complex<R> exp (const complex<R>& z) {
  R r = exp (z.re);
  R x = sin (z.im);
  R y = cos (z.im);
  return complex<R> (x*r, y*r); }
TMPL inline complex<R> log (const complex<R>& z) {
  R r = log (hypot (z.re, z.im));
  R t = atan2 (z.im, z.re);
  return complex<R> (r, t); }
TMPL inline ostream& operator << (ostream& out, const complex<R> z) {
  if (z.re != 0) out << z.re;
  if (z.im >  0) out << "+" << z.im;
  if (z.im <  0) out << z.im;
  return out; }
#undef TMPL

#endif // defined COMPLEX_H
