
/******************************************************************************
* MODULE     : series.gen.cc
* DESCRIPTION: Fast routines on series segments
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef SERIES_CC
#define SERIES_CC

template<class C> void
print (C* series, int n) {
  int i;
  for (i=0; i<n; i++)
    cout << i << ": " << series[i] << "\n";
  cout << "----------------------------------------------\n";
}

template<class C> void
add (C* dest, C* arg1, C* arg2, int n) {
  int i;
  for (i=0; i<n; i++) dest[i]= arg1[i]+ arg2[i];
}

template<class C> void
substract (C* dest, C* arg1, C* arg2, int n) {
  int i;
  for (i=0; i<n; i++) dest[i]= arg1[i]- arg2[i];
}

template<class C> void
multiply (C* dest, C* arg1, C* arg2, int n) {
  int i, j;
  for (i=0; i<n; i++) {
    dest[i]= 0;
    for (j=0; j<=i; j++)
      dest[i] += arg1[j] * arg2[i-j];
  }
}

template<class C> void
divide (C* dest, C* arg1, C* arg2, int n) {
  if (n==1) dest[0]= arg1[0]/arg2[0];
  else {
    int i, mid= (n+1)>>1;
    divide (dest, arg1, arg2, mid);
    for (i=mid; i<n; i++) dest[i]= C(0);
    C temp[n];
    multiply (temp, arg2, dest, n);
    for (i=mid; i<n; i++) temp[i]= arg1[i]- temp[i];
    divide (dest+mid, temp+mid, arg2, n-mid);
  }
}

template<class C> void
logarithm (C* dest, C* arg, int n) {
  int i;
  C temp[n];
  for (i=0; i<n; i++) temp[i]= C(i) * arg[i];
  divide (dest, temp, arg, n);
  for (i=1; i<n; i++) dest[i]= dest[i] / C(i);
  dest[0]= log (arg[0]);
}

template<class C> void
exponentiate (C* dest, C* arg, int n) {
  if (n==1) dest[0]= exp (arg[0]);
  else {
    int i, mid= (n+1)>>1;
    C aux[n];
    exponentiate (aux, arg, mid);
    for (i=mid; i<n; i++) aux[i]= C(0);
    C logf[n];
    logarithm (logf, aux, n);
    C temp[n];
    temp[0]= C(1);
    for (i=1; i<mid; i++) temp[i]= C(0);
    for (i=mid; i<n; i++) temp[i]= arg[i]- logf[i];
    multiply (dest, aux, temp, n);
  }
}

template<class C> C
eval_series (C* series, int n, C z) {
  int i;
  C sum (0);
  for (i= n-1; i>=0; i--)
    sum= z*sum+ series[i];
  return sum;
}

#endif // defined SERIES_CC
