
/******************************************************************************
* MODULE     : mould.gen.cc
* DESCRIPTION: abstract power mould
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef MOULD_CC
#define MOULD_CC
#include "Mould/mould.hh"
#include "hashmap.hh"
#include "Mould/word.hh"

/******************************************************************************
* General routines
******************************************************************************/

template<class C, class G> C
mould<C,G>::operator [] (word<G> w) {
  if (!rep->coeff->contains (w))
    rep->coeff (w)= rep->compute (w);
  return rep->coeff [w];
}

template<class C, class G> void
mould<C,G>::set_precision (int n) {
  mould<C,G>::precision= n;
}

template<class C, class G> ostream&
operator << (ostream& out, mould<C,G> M) {
  int d, i;
  for (d=0; d<mould<C,G>::precision; d++) {
    array<word<G> > a= word<G>::homogeneous (d);
    for (i=0; i<N(a); i++)
      out << a[i] << " -> " << M[a[i]] << "\n";
  }
  return out;
}

/******************************************************************************
* Constant moulds
******************************************************************************/

template<class C, class G> struct constant_mould_rep: public mould_rep<C,G> {
  C c;
  constant_mould_rep (C c, word<G> w) { coeff(w)= c; }
  C compute (word<G> w) { (void) w; return 0; }
};

template<class C, class G> mould<C,G>::mould (C c, word<G> w) {
  rep= new constant_mould_rep<C,G> (c,w);
  rep->ref_count++;
}

/******************************************************************************
* Particular moulds
******************************************************************************/

template<class C, class G> static C
mould<C,G>::factorial (int n) {
  if (n==0) return 1; else return n*factorial (n-1);
}

template<class C, class G> struct zexp_mould_rep : public mould_rep<C,G> {
  C c;
  zexp_mould_rep () { word<G> empty; coeff (empty) = 0; }
  C compute (word<G> w) { return 1/factorial(N(w)); }
};

template<class C, class G> static mould<C,G>
mould<C,G>::zexp () {
  return new zexp_mould_rep<C,G> ();
}

template<class C, class G> struct zlog_mould_rep : public mould_rep<C,G> {
  C c;
  zlog_mould_rep () { word<G> empty; coeff (empty) = 0; }
  C compute (word<G> w) { return (((N(w)&1)==0)? -1/N(w): 1/N(w)); }
};

template<class C, class G> static mould<C,G>
mould<C,G>::zlog () {
  return new zlog_mould_rep<C,G> ();
}

#endif // defined MOULD_CC
