
/******************************************************************************
* MODULE     : mould.gen.h
* DESCRIPTION: abstract power mould
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef MOULD_H
#define MOULD_H
#include "tree.hh"
#include "hashmap.hh"
#include "word.hh"

template<class C, class G> class mould_rep : public abstract_struct {
protected:
  hashmap<word<G>,C> coeff;

  inline mould_rep (): coeff (0) {}
  inline virtual ~mould_rep () {}
  virtual C compute (word<G> w) = 0;

public:
  friend class mould<C,G>;
};

template<class C, class G> struct mould;
template<class C, class G> ostream& operator << (ostream& out, mould<C,G> f);

template<class C, class G> struct mould {
  ABSTRACT_NULL_TEMPLATE_2(mould,C,G);
  mould (C c, word<G> w= word<G> ());
  C operator [] (word<G> w);
  // operator tree ();
  static void set_precision (int n);
  static mould<C,G> zexp ();
  static mould<C,G> zlog ();
private:
  static int precision;
  friend ostream& operator << <> (ostream& out, mould<C,G> f);
protected:
  static C factorial (int n);
};
ABSTRACT_NULL_TEMPLATE_2_CODE(mould,class,C,class,G);

#define MOULD_CODE(C,G) \
template<> int mould<C,G>::precision = 5

// mould<C,G> operator +  (mould<C,G> f, mould<C,G> g);
// mould<C,G> operator -  (mould<C,G> f, mould<C,G> g);
// mould<C,G> operator *  (C c, mould<C,G> f);
// mould<C,G> operator *  (mould<C,G> f, mould<C,G> g);
// mould<C,G> operator /  (mould<C,G> f, mould<C,G> g);

#include "Mould/mould.cc"
#endif // defined MOULD_H
