
/******************************************************************************
* MODULE     : series.gen.h
* DESCRIPTION: abstract power series
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef SERIES_H
#define SERIES_H
#include "Univariate/zealous.hh"
#include "tree.hh"

#define POLYNOMIAL   0
#define RATIONAL     1
#define ALGEBRAIC    2
#define HOLONOMIC    3
#define ZEALOUS      4
#define RELAXED      5


template<class C> struct series;
template<class C> ostream& operator << (ostream& out, series<C> f);

template<class C> class series_rep : public abstract_struct {
protected:
  int        type;   // one of the above types
  int        n;      // number of computed terms
  zealous<C> value;  // truncated series with computed terms

  inline series_rep (int type2): type (type2), n (0), value (0) {}
  inline virtual ~series_rep () {}
  virtual C next () = 0;
  virtual operator tree () = 0;

public:
  inline virtual void set_precision (int terms) { (void) terms; }
  friend class series<C>;
};

template<class C> struct series {
  ABSTRACT_NULL_TEMPLATE(series,C);
  series (C c, int deg=0);
  series (zealous<C> s);
  series (C (*coeffs) (int));
  C operator [] (int n);
  zealous<C> operator () (int start, int end);
  operator tree ();

  static void set_print_precision (int n);
  friend ostream& operator << <> (ostream& out, series<C> f);
private:
  static int print_precision;
};
ABSTRACT_NULL_TEMPLATE_CODE(series,class,C);

#define SERIES_CODE(C,n) \
template<> int series<C>::print_precision = n

#define TMPL template<class C>
TMPL series<C> operator >> (series<C> f, int n);
TMPL series<C> operator << (series<C> f, int n);
TMPL series<C> operator +  (series<C> f, series<C> g);
TMPL series<C> operator -  (series<C> f, series<C> g);
TMPL series<C> operator *  (C c, series<C> f);
TMPL series<C> operator *  (series<C> f, series<C> g);
TMPL series<C> operator /  (series<C> f, series<C> g);

TMPL series<C> d (series<C> f);

// if has_divisible<C>()
TMPL series<C> operator / (series<C> f, C c);
TMPL series<C> I (series<C> f);
TMPL series<C> log (series<C> f);
TMPL series<C> exp (series<C> f);
// endif

TMPL series<C> Mahler  (series<C> f, int p);
TMPL series<C> compose (series<C> f, zealous<C> g);
TMPL series<C> compose (series<C> f, zealous<C> num, zealous<C> den);
TMPL series<C> compose (series<C> f, series<C> g);
TMPL series<C> reverse (series<C> f);

#endif // defined SERIES_H
