
/******************************************************************************
* MODULE     : zealous.gen.h
* DESCRIPTION: zealous power series
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef ZEALOUS_H
#define ZEALOUS_H
#include "basic.hh"
#include "category.hh"

template<class C> inline C ZERO () { return 0; }
template<class C> inline C ONE () { return 1; }

template<class C> class zealous;
template<class C> int N (zealous<C> f);

template<class C> class zealous_rep : concrete_struct {
  C*  a;   // the elements
  int n;   // the length of the truncated power series

  inline zealous_rep (int n2):
    a (n2==0? (C*) NULL: new C[n2](ZERO<C>())), n (n2) {}
  inline ~zealous_rep () { if (n>0) delete[] a; }

public:
  friend class zealous<C>;
  void   accumulate (zealous<C> f, int at=0);
  friend int N<> (zealous<C> f);
  friend zealous<C> multiply<> (zealous<C> f, zealous<C> g);
  friend zealous<C> multiply<> (zealous<C> f, zealous<C> g, int order);
  friend zealous<C> operator * <> (zealous<C> f, zealous<C> g);
};

template<class C> class zealous {
  CONCRETE_TEMPLATE(zealous,C);
  inline zealous () { rep= new zealous_rep<C> (0); }
  inline zealous (int n) { rep= new zealous_rep<C> (n); }
  inline C& operator [] (int i) { return rep->a[i]; }
  zealous<C> operator () (int start, int end);
};
CONCRETE_TEMPLATE_CODE(zealous,class,C);

template<class C> inline int N (zealous<C> f) { return f->n; }

template<class C> struct has_divisible<zealous<C> > :
    define_bool<has_divisible<C>::value> {};
template<class C> struct has_special_functions<zealous<C> > : 
    define_bool<has_divisible<C>::value> {};

#define TMPL template<class C>
TMPL zealous<C> constant    (C c, int k);
TMPL zealous<C> id          (int n);
TMPL zealous<C> operator << (zealous<C> f, int k);
TMPL zealous<C> operator >> (zealous<C> f, int k);

TMPL zealous<C> operator + (zealous<C> f, zealous<C> g);
TMPL zealous<C> operator - (zealous<C> f, zealous<C> g);
TMPL zealous<C> operator * (C c, zealous<C> f);
TMPL zealous<C> multiply   (zealous<C> f, zealous<C> g);
TMPL zealous<C> multiply   (zealous<C> f, zealous<C> g, int order);
TMPL zealous<C> operator * (zealous<C> f, zealous<C> g);
TMPL zealous<C> inv        (zealous<C> f);
TMPL zealous<C> operator / (zealous<C> f, zealous<C> g);

// if has_divisible<C>()
TMPL zealous<C> operator / (zealous<C> f, C c);
TMPL zealous<C> d   (zealous<C> f);
TMPL zealous<C> I   (zealous<C> f);
TMPL zealous<C> log (zealous<C> f);
TMPL zealous<C> exp (zealous<C> f);
// endif

TMPL zealous<C> compose (zealous<C> f, zealous<C> g);
TMPL zealous<C> reverse (zealous<C> f);

TMPL ostream& operator << (ostream& out, zealous<C> f);
#undef TMPL

#include "Univariate/Zealous/zealous.cc"
#endif // defined ZEALOUS_H
