
/******************************************************************************
* MODULE     : canvas_test.gen.cc
* DESCRIPTION: Track keyboard and mouse events
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "window.hh"
#include "Widget/basic_widget.hh"

/******************************************************************************
* Test widget
******************************************************************************/

class track_widget_rep: public basic_widget_rep {
public:
  track_widget_rep (display dis);
  operator tree ();
  void handle_get_size (get_size_event ev);
  void handle_repaint (repaint_event ev);
  void handle_keypress (keypress_event ev);
  void handle_mouse (mouse_event ev);
};

track_widget_rep::track_widget_rep (display dis): basic_widget_rep (dis) {}

widget
track_widget (display dis) {
  return new track_widget_rep (dis);
}

track_widget_rep::operator tree () {
  return "test";
}

void
track_widget_rep::handle_get_size (get_size_event ev) {
  if (ev->mode == 0) {
    ev->w=500*PIXEL;
    ev->h=300*PIXEL;
  }
  else basic_widget_rep::handle_get_size (ev);
}

void
track_widget_rep::handle_repaint (repaint_event ev) { (void) ev;
  win->set_color (dis->white);
  win->fill (ev->x1, ev->y1, ev->x2, ev->y2);

  int i;
  for (i=0; i<=63; i++) {
    int r= (i&3)*85;
    int g= ((i>>2)&3)*85;
    int b= ((i>>4)&3)*85;
    win->set_line_style (2*PIXEL);
    win->set_color (dis->rgb (r,g,b));
    win->line (0, 0, (i*w)/64, -h);
  }
}

void
track_widget_rep::handle_keypress (keypress_event ev) {
  cout << ((event) ev) << "\n";
}

void
track_widget_rep::handle_mouse (mouse_event ev) {
  cout << ((event) ev) << "\n";
}

/******************************************************************************
* The main program
******************************************************************************/

int
main (int argc, char** argv) {
  display dis= open_display (argc, argv);
  widget wid= canvas_widget (track_widget (dis));
  wid << set_extents (0, -2000*PIXEL, 2000*PIXEL, 0);
  window w1= plain_window (wid, "canvas test");
			    
  w1->map ();
  dis->event_loop ();
  close_display (dis);
  return 0;
}

